#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <limits.h>

typedef struct _node {
   unsigned long   val;
   struct _node   *next;
}               NODE;

unsigned long
delta(unsigned long from, unsigned long amt)
{
   if (from < amt) {
      unsigned long long ull = ULONG_MAX + 1 + from - amt;
      return ((unsigned long) ull & ULONG_MAX);
   } else
      return (from - amt);
}

int
ulcmp(const void *a, const void *b)
{
   unsigned long  *x = (unsigned long *) a;
   unsigned long  *y = (unsigned long *) b;

   if (*x == *y)
      return 0;
   else if (*x < *y)
      return -1;
   else
      return 1;
}

int
main()
{
   NODE           *list = 0, *this;
   unsigned int    lines = 0;
   unsigned long   in, out, when;
   unsigned long   last_in, last_out, last_when, dummy;
   unsigned long  *arr;
   size_t          linelen;
   char           *line, buf[80];
   int             fields;

   while (!feof(stdin) && (line = fgetln(stdin, &linelen))) {

      strncpy(buf, line, linelen);
      buf[--linelen] = 0;

      if ((fields = sscanf(buf, "%lu %lu %lu %lu\n",
			   &when, &in, &dummy, &out)) != 4) {
	 printf("hiccup: fields = %d, lines = %d, line = %s, len = %d\n",
		fields, lines, buf, linelen);
	 break;
      }
      if (++lines > 1) {
	 assert(this = malloc(sizeof(NODE)));
	 this->val = delta(in, last_in) + delta(out, last_out);
         this->val /= delta(when, last_when);

	 fprintf(stderr, "%lu\n", this->val);

	 this->next = (list ? list : 0);
	 list = this;
      }
      last_in = in;
      last_out = out;
      last_when = when;
   }

   assert(arr = malloc(sizeof(unsigned long) * (lines - 1)));

   lines = 0;
   this = list;
   while (this) {
      arr[lines++] = this->val;
      this = this->next;
   }

   qsort((void *) arr, lines, sizeof(unsigned long), ulcmp);

   printf("min = %lu\n", arr[0]);
   printf("max = %lu\n", arr[lines - 1]);

}
