#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <assert.h>
#include <gmp.h>

unsigned long
delta(unsigned long from, unsigned long amt)
{
   if (from < amt) {
      unsigned long long ull = ULONG_MAX + 1 + from - amt;
      return ((unsigned long) ull & ULONG_MAX);
   } else
      return (from - amt);
}

int
main()
{
   mpz_t           ttl_in, ttl_out;
   unsigned long   in, out;
   unsigned long   last_in, last_out, d1, d2;
   unsigned int    lines = 0;
   size_t          linelen;
   char           *line, buf[80];
   time_t          start = 0, end;
   unsigned long   duration;

   mpz_init(ttl_in);
   mpz_init(ttl_out);

   while (!feof(stdin) && (line = fgetln(stdin, &linelen))) {

      strncpy(buf, line, linelen);
      buf[--linelen] = 0;

      assert((sscanf(buf, "%lu %lu %lu %lu\n",
		     &d1, &in, &d2, &out)) == 4);

      if (++lines > 1) {
	 mpz_add_ui(ttl_in, ttl_in, delta(in, last_in));
	 mpz_add_ui(ttl_out, ttl_out, delta(out, last_out));
      } else
	 start = d1;

      last_in = in;
      last_out = out;
      end = d2;
   }

   duration = end - start;

   printf("\ntotal in  = ");
   mpz_out_str(stdout, 10, ttl_in);
   printf("\ntotal out = ");
   mpz_out_str(stdout, 10, ttl_out);
   printf("\nduration  = %lu\n", duration);

   mpz_cdiv_q_ui(ttl_in, ttl_in, duration);
   mpz_cdiv_q_ui(ttl_out, ttl_out, duration);

   printf("\naverage in  = ");
   mpz_out_str(stdout, 10, ttl_in);
   printf("\naverage out = ");
   mpz_out_str(stdout, 10, ttl_out);
   printf("\n\n");

}
