#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <limits.h>

typedef struct _node {
   unsigned long   in;
   unsigned long   out;
   unsigned long   agg;
   struct _node   *next;
}               NODE;

unsigned long
delta(unsigned long from, unsigned long amt)
{
   if (from < amt) {
      unsigned long long ull = ULONG_MAX + 1 + from - amt;
      return ((unsigned long) ull & ULONG_MAX);
   } else
      return (from - amt);
}

int
ulcmp(const void *a, const void *b)
{
   unsigned long  *x = (unsigned long *) a;
   unsigned long  *y = (unsigned long *) b;

   if (*x == *y)
      return 0;
   else if (*x < *y)
      return -1;
   else
      return 1;
}

int
main()
{
   NODE           *list = 0, *this;
   unsigned int    lines = 0;
   unsigned long   in, out, when, pct_smp;
   unsigned long   last_in, last_out, last_when, dummy;
   unsigned long  *ina, *outa, *agga;
   size_t          linelen;
   char           *line, buf[80];
   int             fields;

   while (!feof(stdin) && (line = fgetln(stdin, &linelen))) {

      strncpy(buf, line, linelen);
      buf[--linelen] = 0;

      if ((fields = sscanf(buf, "%lu %lu %lu %lu\n",
			   &when, &in, &dummy, &out)) != 4) {
	 printf("hiccup: fields = %d, lines = %d, line = %s, len = %d\n",
		fields, lines, buf, linelen);
	 break;
      }
      if (++lines > 1) {
	 assert(this = malloc(sizeof(NODE)));
	 this->in = delta(in, last_in) / delta(when, last_when);
	 this->out = delta(out, last_out) / delta(when, last_when);
	 this->agg = this->in + this->out / delta(when, last_when);

	 // fprintf(stderr, "%lu\n", this->val);

	 this->next = (list ? list : 0);
	 list = this;
      }
      last_in = in;
      last_out = out;
      last_when = when;
   }

   assert(ina = malloc(sizeof(NODE) * (lines - 1)));
   assert(outa = malloc(sizeof(NODE) * (lines - 1)));
   assert(agga = malloc(sizeof(NODE) * (lines - 1)));

   lines = 0;
   this = list;
   while (this) {
      ina[lines] = this->in;
      outa[lines] = this->out;
      agga[lines++] = this->agg;
      this = this->next;
   }

   qsort((void *) ina, lines, sizeof(unsigned long), ulcmp);
   qsort((void *) outa, lines, sizeof(unsigned long), ulcmp);
   qsort((void *) agga, lines, sizeof(unsigned long), ulcmp);

   //pct_smp = (lines / 100) * 95;
   pct_smp = lines * 0.95;

   printf("samples = %lu, 95%% = %lu\n",  lines, pct_smp);

   printf("in[%lu] = %lu, ", pct_smp, ina[pct_smp]);
   printf("out[%lu] = %lu, ", pct_smp, outa[pct_smp]);
   printf("agg[%lu] = %lu\n", pct_smp, agga[pct_smp]);

}
